<?php

/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\Newsletter\Controller\Subscriber;

use Exception;
use FiloBlu\Newsletter\Helper\Data;
use FiloBlu\Newsletter\Model\PreConfirm;
use FiloBlu\Newsletter\Model\ThankYou;
use Magento\Customer\Api\AccountManagementInterface as CustomerAccountManagement;
use Magento\Customer\Model\Session;
use Magento\Customer\Model\Url as CustomerUrl;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Newsletter\Model\Subscriber;
use Magento\Newsletter\Model\SubscriberFactory;
use Magento\Store\Model\StoreManagerInterface;

class NewLandingAction extends \Magento\Newsletter\Controller\Subscriber\NewAction
{
    public $_helper;
    protected $jsonResultFactory;
    protected $_modelPreConfirm;
    protected $_modelThankYou;
    /**
     * @var JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * NewLandingAction constructor.
     * @param Context $context
     * @param SubscriberFactory $subscriberFactory
     * @param Session $customerSession
     * @param StoreManagerInterface $storeManager
     * @param CustomerUrl $customerUrl
     * @param CustomerAccountManagement $customerAccountManagement
     * @param JsonFactory $resultJsonFactory
     * @param Data $helper
     * @param PreConfirm $modelPreConfirm
     * @param ThankYou $modelThankYou
     */
    public function __construct(
        Context $context,
        SubscriberFactory $subscriberFactory,
        Session $customerSession,
        StoreManagerInterface $storeManager,
        CustomerUrl $customerUrl,
        CustomerAccountManagement $customerAccountManagement,
        JsonFactory $resultJsonFactory,
        Data $helper,
        PreConfirm $modelPreConfirm,
        ThankYou $modelThankYou
    )
    {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->_helper = $helper;
        $this->_modelPreConfirm = $modelPreConfirm;
        $this->_modelThankYou = $modelThankYou;

        parent::__construct(
            $context,
            $subscriberFactory,
            $customerSession,
            $storeManager,
            $customerUrl,
            $customerAccountManagement
        );
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json|\Magento\Framework\Controller\Result\Redirect|null
     */
    public function execute()
    {
        if ($this->_helper->getConfig('general/enabled') && $this->getRequest()->isPost() && $this->getRequest()->getPost('email')) {
            $resultJson = $this->resultJsonFactory->create();

            $email = (string)$this->getRequest()->getPost('email');

            try {
                $this->validateEmailFormat($email);
                $this->validateGuestSubscription();
                $this->validateEmailAvailable($email);

                $status = $this->_subscriberFactory->create()->subscribe($email);

                if ($status == Subscriber::STATUS_NOT_ACTIVE) {
                    if ($this->getRequest()->getPost('preconfirm_id') && $this->getRequest()->getPost('preconfirm_id') != 0) {
                        $this->_helper->setPreConfirmCookie($this->getRequest()->getPost('preconfirm_id'));
                        $resultJson->setData(['success' => true, 'error' => false, 'redirect' => $this->_storeManager->getStore()->getBaseUrl() . $this->_modelPreConfirm->getUrlPreConfirmPageById($this->getRequest()->getPost('preconfirm_id'))->getData()["url_key"]]);
                    } else {
//                        $this->messageManager->addSuccess(__('The confirmation request has been sent.'));
                        $resultJson->setData(['success' => true, 'error' => false, 'message' => __("The confirmation request has been sent."), 'redirect' => $this->_storeManager->getStore()->getBaseUrl()]);
                    }
                } else {
                    $subscriber = $this->_subscriberFactory->create()->loadByEmail($email);
                    if ($this->getRequest()->getPost('thankyou_id') && $this->getRequest()->getPost('thankyou_id') != 0) {
                        $this->_helper->setThankYouCookie($this->getRequest()->getPost('thankyou_id'));
                        $resultJson->setData(['subscriber_id' => $subscriber->getId(), 'success' => true, 'error' => false, 'redirect' => $this->_storeManager->getStore()->getBaseUrl() . $this->_modelThankYou->getUrlThankYouPageById($this->getRequest()->getPost('thankyou_id'))->getData()["url_key"]]);
                    } else {
//                        $this->messageManager->addSuccess(__('Thank you for your subscription.'));
                        $resultJson->setData(['subscriber_id' => $subscriber->getId(), 'success' => true, 'error' => false, 'message' => __("Thank you for your subscription."), 'redirect' => $this->_storeManager->getStore()->getBaseUrl()]);
                    }
                }
            } catch (LocalizedException $e) {
                $resultJson->setData(['success' => false, 'error' => true, 'message' => __('There was a problem with the subscription: %1', $e->getMessage())]);
            } catch (Exception $e) {
                $resultJson->setData(['success' => false, 'error' => true, 'message' => __($e->getMessage())]);
            }

            return $resultJson;
        }

        return null;
    }

    /**
     * Validates that the email address isn't being used by a different account.
     *
     * @param string $email
     * @return void
     * @throws LocalizedException
     */
    protected function validateEmailAvailable($email)
    {
        $websiteId = $this->_storeManager->getStore()->getWebsiteId();
        if ($this->_customerSession->getCustomerDataObject()->getEmail() !== $email && !$this->customerAccountManagement->isEmailAvailable($email, $websiteId)
        ) {
            throw new LocalizedException(
                __('This email address is already assigned to another user.')
            );
        }

        $subscriber = $this->_subscriberFactory->create()->loadByEmail($email);

        if ($subscriber->getSubscriberStatus() == Subscriber::STATUS_SUBSCRIBED) {
            throw new LocalizedException(
                __('This email address is already assigned to another user.')
            );
        }
    }
}
